@@ -112,7 +112,7 @@ class BlockVolumeManifest(volume.VolumeManifest):
             raise se.VolumeMetadataReadError("%s: %s" % (metaId, e))
 
         md = VolumeMetadata.from_lines(lines)
-        return md.legacy_info()
+        return md.legacy_info(sd.getVersion())
 
     def validateImagePath(self):
         """
@@ -507,7 +507,7 @@ class BlockVolume(volume.Volume):
         lvSize = cls.calculate_volume_alloc_size(preallocate,
                                                  size, initialSize)
 
-        lvm.createLV(dom.sdUUID, volUUID, "%s" % lvSize, activate=True,
+        lvm.createLV(dom.sdUUID, volUUID, lvSize, activate=True,
                      initialTags=(sc.TAG_VOL_UNINIT,))
 
         fileutils.rm_file(volPath)
@@ -532,8 +532,7 @@ class BlockVolume(volume.Volume):
                          imgUUID, volUUID, srcImgUUID, srcVolUUID, size)
             volParent.clone(volPath, volFormat, size)
 
-        with dom.acquireVolumeMetadataSlot(
-                volUUID, sc.VOLUME_MDNUMBLKS) as slot:
+        with dom.acquireVolumeMetadataSlot(volUUID) as slot:
             mdTags = ["%s%s" % (sc.TAG_PREFIX_MD, slot),
                       "%s%s" % (sc.TAG_PREFIX_PARENT, srcVolUUID),
                       "%s%s" % (sc.TAG_PREFIX_IMAGE, imgUUID)]
