@@ -223,7 +223,7 @@ class VolumeManifest(object):
             "size": int(meta.get(sc.SIZE, "0")),
             "parent": self.getParent(),
             "description": meta.get(sc.DESCRIPTION, ""),
-            "pool": meta.get(sc.POOL, ""),
+            "pool": "",  # deprecated value
             "domain": meta.get(sc.DOMAIN, ""),
             "image": self.getImage(),
             "ctime": meta.get(sc.CTIME, ""),
@@ -533,7 +533,8 @@ class VolumeManifest(object):
                     voltype, disktype, desc="", legality=sc.ILLEGAL_VOL):
         meta = VolumeMetadata(sdUUID, imgUUID, puuid, size, format, type,
                               voltype, disktype, desc, legality)
-        cls.createMetadata(metaId, meta.legacy_info())
+        sd = sdCache.produce_manifest(sdUUID)
+        cls.createMetadata(metaId, meta.legacy_info(sd.getVersion()))
         return meta
 
     def refreshVolume(self):
@@ -1137,8 +1138,8 @@ class Volume(object):
                             in case of thin provisioning
         """
         dom = sdCache.produce(sdUUID)
-        dom.validateCreateVolumeParams(volFormat, srcVolUUID,
-                                       preallocate=preallocate)
+        dom.validateCreateVolumeParams(
+            volFormat, srcVolUUID, diskType=diskType, preallocate=preallocate)
 
         imgPath = dom.create_image(imgUUID)
 
