@@ -23,7 +23,9 @@ from __future__ import absolute_import
 import logging
 import time
 
-from vdsm.storage import constants
+import six
+
+from vdsm.storage import constants as sc
 from vdsm.storage import exception
 
 
@@ -31,14 +33,17 @@ class VolumeMetadata(object):
 
     log = logging.getLogger('storage.VolumeMetadata')
 
-    def __init__(self, domain, image, puuid, size, format,
-                 type, voltype, disktype, description="",
-                 legality=constants.ILLEGAL_VOL, ctime=None, mtime=None,
-                 generation=constants.DEFAULT_GENERATION):
-        assert(isinstance(size, int))
-        assert(ctime is None or isinstance(ctime, int))
-        assert(mtime is None or isinstance(mtime, int))
-        assert(isinstance(generation, int))
+    def __init__(self, domain, image, puuid, size, format, type, voltype,
+                 disktype, description="", legality=sc.ILLEGAL_VOL, ctime=None,
+                 generation=sc.DEFAULT_GENERATION):
+        if not isinstance(size, six.integer_types):
+            raise AssertionError("Invalid value for 'size': {!r}".format(size))
+        if ctime is not None and not isinstance(ctime, int):
+            raise AssertionError(
+                "Invalid value for 'ctime': {!r}".format(ctime))
+        if not isinstance(generation, int):
+            raise AssertionError(
+                "Invalid value for 'generation': {!r}".format(generation))
 
         # Storage domain UUID
         self.domain = domain
@@ -62,8 +67,6 @@ class VolumeMetadata(object):
         self.legality = legality
         # Volume creation time (in seconds since the epoch)
         self.ctime = int(time.time()) if ctime is None else ctime
-        # Volume modification time (unused and should be zero)
-        self.mtime = 0 if mtime is None else mtime
         # Generation increments each time certain operations complete
         self.generation = generation
 
@@ -79,25 +82,24 @@ class VolumeMetadata(object):
             md[key.strip()] = value.strip()
 
         try:
-            return cls(domain=md[constants.DOMAIN],
-                       image=md[constants.IMAGE],
-                       puuid=md[constants.PUUID],
-                       size=int(md[constants.SIZE]),
-                       format=md[constants.FORMAT],
-                       type=md[constants.TYPE],
-                       voltype=md[constants.VOLTYPE],
-                       disktype=md[constants.DISKTYPE],
-                       description=md[constants.DESCRIPTION],
-                       legality=md[constants.LEGALITY],
-                       ctime=int(md[constants.CTIME]),
-                       mtime=int(md[constants.MTIME]),
+            return cls(domain=md[sc.DOMAIN],
+                       image=md[sc.IMAGE],
+                       puuid=md[sc.PUUID],
+                       size=int(md[sc.SIZE]),
+                       format=md[sc.FORMAT],
+                       type=md[sc.TYPE],
+                       voltype=md[sc.VOLTYPE],
+                       disktype=md[sc.DISKTYPE],
+                       description=md[sc.DESCRIPTION],
+                       legality=md[sc.LEGALITY],
+                       ctime=int(md[sc.CTIME]),
                        # generation was added to the set of metadata keys well
                        # after the above fields.  Therefore, it may not exist
                        # on storage for pre-existing volumes.  In that case we
                        # report a default value of 0 which will be written to
                        # the volume metadata on the next metadata change.
-                       generation=int(md.get(constants.GENERATION,
-                                             constants.DEFAULT_GENERATION)))
+                       generation=int(md.get(sc.GENERATION,
+                                             sc.DEFAULT_GENERATION)))
         except KeyError as e:
             raise exception.MetaDataKeyNotFoundError(
                 "Missing metadata key: %s: found: %s" % (e, md))
@@ -116,44 +118,56 @@ class VolumeMetadata(object):
         # We cannot fail when the description is too long, since we must
         # support older engine that may send such values, or old disks
         # with long description.
-        if len(desc) > constants.DESCRIPTION_SIZE:
+        if len(desc) > sc.DESCRIPTION_SIZE:
             cls.log.warning("Description is too long, truncating to %d bytes",
-                            constants.DESCRIPTION_SIZE)
-            desc = desc[:constants.DESCRIPTION_SIZE]
+                            sc.DESCRIPTION_SIZE)
+            desc = desc[:sc.DESCRIPTION_SIZE]
         return desc
 
-    def storage_format(self):
+    def storage_format(self, domain_version):
         """
         Format metadata string in storage format.
 
         Raises MetadataOverflowError if formatted metadata is too long.
+
+        NOTE: Not used yet! We need to drop legacy_info() and pass
+        VolumeMetadata instance instead of a dict to use this code.
         """
-        info = self.legacy_info()
+        info = self.legacy_info(domain_version)
         keys = sorted(info.keys())
         lines = ["%s=%s\n" % (key, info[key]) for key in keys]
         lines.append("EOF\n")
         data = "".join(lines)
-        if len(data) > constants.METADATA_SIZE:
+        if len(data) > sc.METADATA_SIZE:
             raise exception.MetadataOverflowError(data)
         return data
 
-    def legacy_info(self):
+    def legacy_info(self, domain_version):
         """
         Return metadata in dictionary format
+
+        NOTE: The dict returned here will be formatted by
+        Volume.formatMetadata() and written to storage.
         """
-        return {
-            constants.FORMAT: self.format,
-            constants.TYPE: self.type,
-            constants.VOLTYPE: self.voltype,
-            constants.DISKTYPE: self.disktype,
-            constants.SIZE: str(self.size),
-            constants.CTIME: str(self.ctime),
-            constants.POOL: "",  # obsolete
-            constants.DOMAIN: self.domain,
-            constants.IMAGE: self.image,
-            constants.DESCRIPTION: self.description,
-            constants.PUUID: self.puuid,
-            constants.MTIME: str(self.mtime),
-            constants.LEGALITY: self.legality,
-            constants.GENERATION: self.generation,
+        md = {
+            sc.FORMAT: self.format,
+            sc.TYPE: self.type,
+            sc.VOLTYPE: self.voltype,
+            sc.DISKTYPE: self.disktype,
+            sc.SIZE: str(self.size),
+            sc.CTIME: str(self.ctime),
+            sc.DOMAIN: self.domain,
+            sc.IMAGE: self.image,
+            sc.DESCRIPTION: self.description,
+            sc.PUUID: self.puuid,
+            sc.LEGALITY: self.legality,
+            sc.GENERATION: self.generation,
         }
+        if domain_version < 5:
+            # Always zero on pre v5 domains
+            # We need to keep MTIME available on pre v5
+            # domains, as other code is expecting that
+            # field to exists and will fail without it.
+            md[sc.MTIME] = 0
+
+        return md
