@@ -180,8 +180,9 @@ class FakeDomainManifest(object):
     def refresh(self):
         pass
 
+    @classmethod
     @recorded
-    def validateCreateVolumeParams(self, volFormat, srcVolUUID,
+    def validateCreateVolumeParams(cls, volFormat, srcVolUUID, diskType=None,
                                    preallocate=None):
         pass
 
@@ -220,11 +221,6 @@ class FakeBlockDomainManifest(FakeDomainManifest):
     def readMetadataMapping(self):
         pass
 
-    @classmethod
-    @recorded
-    def metaSize(cls, *args):
-        pass
-
     @classmethod
     @recorded
     def getMetaDataMapping(cls, *args):
@@ -248,7 +244,7 @@ class FakeBlockDomainManifest(FakeDomainManifest):
 
     @recorded
     @contextmanager
-    def acquireVolumeMetadataSlot(self, vol_name, slotSize):
+    def acquireVolumeMetadataSlot(self, vol_name):
         yield
 
     @recorded
@@ -660,6 +656,11 @@ class RedirectionChecker(object):
         getattr(self.source_instance, fn)(*args)
         self.assertEqual([(fn, args, {})], target.__class_calls__)
 
+    def check_classmethod_call_args_kwargs(self, fn, *args, **kwargs):
+        getattr(self.source_instance, fn)(*args, **kwargs)
+        target = getattr(self.source_instance, self.target_name)
+        self.assertEqual([(fn, args, kwargs)], target.__class_calls__)
+
     def assertEqual(self, expected, actual):
         assert actual == expected, "expected: %r got: %r" % (expected, actual)
 
@@ -688,6 +689,10 @@ class DomainTestMixin(object):
         self.checker.check_method('_getRepoPath', (),
                                   [('getRepoPath', (), {})])
 
+    def test_validate_create_volume_params(self):
+        self.checker.check_classmethod_call_args_kwargs(
+            "validateCreateVolumeParams", 1, 2, diskType=3, preallocate=4)
+
     def test_nonexisting_function(self):
         self.assertRaises(AttributeError,
                           self.checker.check_method_call, 'foo')
@@ -740,7 +745,6 @@ class DomainTestMixin(object):
         ['hasVolumeLeases', 0],
         ['refreshDirTree', 0],
         ['refresh', 0],
-        ['validateCreateVolumeParams', 3],
         ['getVolumeLease', 2],
         ['external_leases_path', 0],
     ])
@@ -760,8 +764,8 @@ class TestBlockDomain(DomainTestMixin, VdsmTestCase):
         self.assertEqual(512, self.domain.phyBlkSize)
 
     def test_acquirevolumemetadataslot(self):
-        with self.domain.acquireVolumeMetadataSlot(0, 1):
-            result = [('acquireVolumeMetadataSlot', (0, 1), {})]
+        with self.domain.acquireVolumeMetadataSlot(0):
+            result = [('acquireVolumeMetadataSlot', (0,), {})]
             self.assertEqual(self.domain._manifest.__calls__, result)
 
     @permutations([
@@ -774,7 +778,6 @@ class TestBlockDomain(DomainTestMixin, VdsmTestCase):
         self.checker.check_method_call(fn, nargs)
 
     @permutations([
-        ['metaSize', 1],
         ['getMetaDataMapping', 2],
     ])
     def test_block_classmethod(self, fn, nargs=0):
