@@ -19,13 +19,14 @@
 
 from __future__ import absolute_import
 from __future__ import division
+from __future__ import print_function
 
 import textwrap
 import time
 
+import pytest
+
 from testlib import make_uuid
-from testlib import VdsmTestCase, permutations, expandPermutations
-from monkeypatch import MonkeyPatchScope
 
 from vdsm.storage import constants as sc
 from vdsm.storage import exception as se
@@ -66,9 +67,7 @@ def make_md_dict(**kwargs):
         sc.DISKTYPE: 'disktype',
         sc.DESCRIPTION: 'description',
         sc.LEGALITY: 'legality',
-        sc.MTIME: '0',
         sc.CTIME: '0',
-        sc.POOL: '',
         sc.GENERATION: '1',
     }
     res.update(kwargs)
@@ -84,10 +83,9 @@ def make_lines(**kwargs):
     return lines
 
 
-@expandPermutations
-class TestVolumeMetadata(VdsmTestCase):
+class TestVolumeMetadata:
 
-    def test_create_info(self):
+    def test_create_info_v4(self, monkeypatch):
         params = make_init_params()
         expected = dict(
             CTIME=str(FAKE_TIME),
@@ -97,19 +95,38 @@ class TestVolumeMetadata(VdsmTestCase):
             FORMAT=params['format'],
             IMAGE=params['image'],
             LEGALITY=params['legality'],
-            MTIME="0",
-            POOL_UUID="",
+            MTIME=0,
             PUUID=params['puuid'],
             SIZE=str(params['size']),
             TYPE=params['type'],
             VOLTYPE=params['voltype'],
             GEN=params['generation'])
 
-        with MonkeyPatchScope([[time, 'time', lambda: FAKE_TIME]]):
-            info = volume.VolumeMetadata(**params).legacy_info()
-            self.assertEqual(expected, info)
+        monkeypatch.setattr(time, 'time', lambda: FAKE_TIME)
+        info = volume.VolumeMetadata(**params).legacy_info(4)
+        assert expected == info
 
-    def test_storage_format(self):
+    def test_create_info_v5(self, monkeypatch):
+        params = make_init_params()
+        expected = dict(
+            CTIME=str(FAKE_TIME),
+            DESCRIPTION=params['description'],
+            DISKTYPE=params['disktype'],
+            DOMAIN=params['domain'],
+            FORMAT=params['format'],
+            IMAGE=params['image'],
+            LEGALITY=params['legality'],
+            PUUID=params['puuid'],
+            SIZE=str(params['size']),
+            TYPE=params['type'],
+            VOLTYPE=params['voltype'],
+            GEN=params['generation'])
+
+        monkeypatch.setattr(time, 'time', lambda: FAKE_TIME)
+        info = volume.VolumeMetadata(**params).legacy_info(5)
+        assert expected == info
+
+    def test_storage_format_v4(self):
         params = make_init_params(ctime=FAKE_TIME)
         expected = textwrap.dedent("""\
             CTIME=%(ctime)s
@@ -121,7 +138,6 @@ class TestVolumeMetadata(VdsmTestCase):
             IMAGE=%(image)s
             LEGALITY=%(legality)s
             MTIME=0
-            POOL_UUID=
             PUUID=%(puuid)s
             SIZE=%(size)s
             TYPE=%(type)s
@@ -129,68 +145,164 @@ class TestVolumeMetadata(VdsmTestCase):
             EOF
             """ % params)
         md = volume.VolumeMetadata(**params)
-        self.assertEqual(expected, md.storage_format())
+        assert expected == md.storage_format(4)
 
-    @permutations([
-        [sc.DESCRIPTION_SIZE],
-        [sc.DESCRIPTION_SIZE + 1]
-    ])
-    def test_long_description(self, size):
-        params = make_init_params(description="!" * size)
+    def test_storage_format_v5(self):
+        params = make_init_params(ctime=FAKE_TIME)
+        expected = textwrap.dedent("""\
+            CTIME=%(ctime)s
+            DESCRIPTION=%(description)s
+            DISKTYPE=%(disktype)s
+            DOMAIN=%(domain)s
+            FORMAT=%(format)s
+            GEN=%(generation)s
+            IMAGE=%(image)s
+            LEGALITY=%(legality)s
+            PUUID=%(puuid)s
+            SIZE=%(size)s
+            TYPE=%(type)s
+            VOLTYPE=%(voltype)s
+            EOF
+            """ % params)
         md = volume.VolumeMetadata(**params)
-        self.assertEqual(sc.DESCRIPTION_SIZE, len(md.description))
+        assert expected == md.storage_format(5)
 
-    @permutations([['size'], ['ctime'], ['mtime']])
+    @pytest.mark.parametrize("param", ['size', 'ctime'])
     def test_int_params_str_raises(self, param):
         params = make_init_params(**{param: 'not_an_int'})
-        self.assertRaises(AssertionError, volume.VolumeMetadata, **params)
+        with pytest.raises(AssertionError):
+            volume.VolumeMetadata(**params)
 
-    @permutations([[key] for key in make_md_dict()
-                   if key not in (sc.POOL, sc.GENERATION)])
+    @pytest.mark.parametrize("required_key",
+                             [key for key in make_md_dict()
+                              if key != sc.GENERATION])
     def test_from_lines_missing_key(self, required_key):
-        data = make_md_dict(POOL=None)
+        data = make_md_dict()
         data[required_key] = None
         lines = make_lines(**data)
-        self.assertRaises(se.MetaDataKeyNotFoundError,
-                          volume.VolumeMetadata.from_lines, lines)
+        with pytest.raises(se.MetaDataKeyNotFoundError):
+            volume.VolumeMetadata.from_lines(lines)
 
-    @permutations([[None], ['pool']])
-    def test_deprecated_pool(self, val):
-        lines = make_lines(**{sc.POOL: val})
-        md = volume.VolumeMetadata.from_lines(lines)
-        self.assertEqual("", md.legacy_info()[sc.POOL])
-
-    def test_from_lines_invalid_param(self):
+    @pytest.mark.parametrize("version", [4, 5])
+    def test_from_lines_invalid_param(self, version):
         lines = make_lines(INVALID_KEY='foo')
-        self.assertNotIn("INVALID_KEY",
-                         volume.VolumeMetadata.from_lines(lines).legacy_info())
+        md = volume.VolumeMetadata.from_lines(lines)
+        assert "INVALID_KEY" not in md.legacy_info(version)
 
-    @permutations([[sc.SIZE], [sc.CTIME], [sc.MTIME]])
+    @pytest.mark.parametrize("key", [sc.SIZE, sc.CTIME])
     def test_from_lines_int_parse_error(self, key):
         lines = make_lines(**{key: 'not_an_integer'})
-        self.assertRaises(ValueError,
-                          volume.VolumeMetadata.from_lines, lines)
+        with pytest.raises(ValueError):
+            volume.VolumeMetadata.from_lines(lines)
 
     def test_from_lines(self):
         data = make_md_dict()
         lines = make_lines(**data)
 
         md = volume.VolumeMetadata.from_lines(lines)
-        self.assertEqual(data[sc.DOMAIN], md.domain)
-        self.assertEqual(data[sc.IMAGE], md.image)
-        self.assertEqual(data[sc.PUUID], md.puuid)
-        self.assertEqual(int(data[sc.SIZE]), md.size)
-        self.assertEqual(data[sc.FORMAT], md.format)
-        self.assertEqual(data[sc.TYPE], md.type)
-        self.assertEqual(data[sc.VOLTYPE], md.voltype)
-        self.assertEqual(data[sc.DISKTYPE], md.disktype)
-        self.assertEqual(data[sc.DESCRIPTION], md.description)
-        self.assertEqual(int(data[sc.MTIME]), md.mtime)
-        self.assertEqual(int(data[sc.CTIME]), md.ctime)
-        self.assertEqual(data[sc.LEGALITY], md.legality)
-        self.assertEqual(int(data[sc.GENERATION]), md.generation)
+        assert data[sc.DOMAIN] == md.domain
+        assert data[sc.IMAGE] == md.image
+        assert data[sc.PUUID] == md.puuid
+        assert int(data[sc.SIZE]) == md.size
+        assert data[sc.FORMAT] == md.format
+        assert data[sc.TYPE] == md.type
+        assert data[sc.VOLTYPE] == md.voltype
+        assert data[sc.DISKTYPE] == md.disktype
+        assert data[sc.DESCRIPTION] == md.description
+        assert int(data[sc.CTIME]) == md.ctime
+        assert data[sc.LEGALITY] == md.legality
+        assert int(data[sc.GENERATION]) == md.generation
 
     def test_generation_default(self):
         lines = make_lines(GEN=None)
         md = volume.VolumeMetadata.from_lines(lines)
-        self.assertEqual(sc.DEFAULT_GENERATION, md.generation)
+        assert sc.DEFAULT_GENERATION == md.generation
+
+
+class TestMDSize:
+    MAX_DESCRIPTION = "x" * sc.DESCRIPTION_SIZE
+    # We don't think that any one will actually preallocate
+    # 1 PB in near future.
+    MAX_PREALLOCATED_SIZE = 1024**5
+    MAX_VOLUME_SIZE = 2**63 - 1
+
+    @pytest.mark.parametrize('size', [
+        sc.DESCRIPTION_SIZE,
+        sc.DESCRIPTION_SIZE + 1
+    ])
+    def test_long_description(self, size):
+        params = make_init_params(description="!" * size)
+        md = volume.VolumeMetadata(**params)
+        assert sc.DESCRIPTION_SIZE == len(md.description)
+
+    @pytest.mark.parametrize('version', [4, 5])
+    @pytest.mark.parametrize('md_params', [
+        # Preallocated block/file example:
+        #
+        # CTIME=1542308390
+        # FORMAT=RAW
+        # DISKTYPE=ISOF
+        # LEGALITY=ILLEGAL
+        # SIZE=2199023255552
+        # VOLTYPE=LEAF
+        # DESCRIPTION={"DiskAlias":"Fedora-Server-dvd-x86_64-29-1.2.iso", "DiskDescription":"Uploaded disk"} # NOQA: E501 (potentially long line)
+        # IMAGE=bc9d15fa-70eb-40aa-8a2e-e4f27664752f
+        # PUUID=00000000-0000-0000-0000-000000000000
+        # MTIME=0
+        # POOL_UUID=
+        # TYPE=PREALLOCATED
+        # GEN=0
+        # EOF
+        {
+            'size': MAX_PREALLOCATED_SIZE,
+            'type': 'PREALLOCATED'
+        },
+        # Sparse block/file example:
+        #
+        # CTIME=1542308390
+        # FORMAT=RAW
+        # DISKTYPE=ISOF
+        # LEGALITY=ILLEGAL
+        # SIZE=18014398509481984
+        # VOLTYPE=LEAF
+        # DESCRIPTION={"DiskAlias":"Fedora-Server-dvd-x86_64-29-1.2.iso", "DiskDescription":"Uploaded disk"} # NOQA: E501 (potentially long line)
+        # IMAGE=bc9d15fa-70eb-40aa-8a2e-e4f27664752f
+        # PUUID=00000000-0000-0000-0000-000000000000
+        # MTIME=0
+        # POOL_UUID=
+        # TYPE=SPARSE
+        # GEN=0
+        # EOF
+        {
+            'size': MAX_VOLUME_SIZE,
+            'type': 'SPARSE'
+        }
+    ])
+    def test_max_size(self, version, md_params):
+        md = volume.VolumeMetadata(
+            ctime=1440935038,
+            description=self.MAX_DESCRIPTION,
+            disktype="ISOF",
+            domain='75f8a1bb-4504-4314-91ca-d9365a30692b',
+            format="RAW",
+            generation=sc.MAX_GENERATION,
+            image='75f8a1bb-4504-4314-91ca-d9365a30692b',
+            legality='ILLEGAL',
+            # Blank UUID for RAW, can be real UUID for COW.
+            puuid=sc.BLANK_UUID,
+            size=md_params['size'] // 512,
+            type=md_params['type'],
+            voltype='INTERNAL',
+        )
+
+        md_len = len(md.storage_format(version))
+        # Needed for documenting sc.MAX_DESCRIPTION.
+        md_fields = md_len - sc.DESCRIPTION_SIZE
+        md_free = sc.METADATA_SIZE - md_len
+
+        # To see this, run:
+        # tox -e storage-py27 tests/storage/volume_metadata_test.py -- -vs
+        print("version={} type={} length={} fields={} free={}"
+              .format(version, md_params['type'], md_len, md_fields, md_free))
+
+        assert sc.METADATA_SIZE >= md_len
